#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version = 0.07
#pragma IgorVersion = 6.3
#pragma ModuleName=DWcalc
#initFunctionName "Init_DebyeWaller()"
#include "Elements", version>=1.72

Static Constant amu_eV = 931.494061e6	// energy of one amu (eV)
Static Constant hbar = 6.58211928e-16	// eV - sec
Static Constant kB = 8.6173324e-5			// eV / K
Static Constant c = 299792458				// speed of light (m/sec)


Menu "Debye-Waller"
	"Calc a Debye-Waller",DebyeWallerInfo("",NaN,NaN)
	"Show Debye-Waller Panel",MakeDebyeWallerPanel()
End


Function DW_factor_M(T,thetaM,Q,amu)		// calculates the M in exp(-M), no I/O
	Variable T			// Temperature (K)
	Variable thetaM	// Debye Temperature (K)
	Variable Q			// length of q vector (1/)
	Variable amu		// mass of atom (amu)
	if (T<0 || thetaM<0 || Q<0 || amu<0 || numtype(T+thetaM+Q+amu))
		return NaN
	elseif (T<=0)
		return 0
	endif

	Variable xx = thetaM / T
	Variable Phi = Integrate1D(DWcalc#PhiIntegrand,0,xx)/xx
	Variable B = (3*hbar^2 * T * c^2)/(2*(amu*amu_eV)*kB*thetaM^2)* (Phi + xx/4)
	Variable M = B *(Q*1.e10)^2		// Q is in 1/, but we need it in 1/m
	return M
End
//
Static Function PhiIntegrand(xx)		// this function should never be called with xx<0
	Variable xx
	return xx>0 ? xx/(exp(xx)-1) : 1
End



Function DebyeWallerInfo(element,T_K,Q_Ang)
	String element
	Variable T_K					// Temperature (K)
	Variable Q_Ang				// length of Q vector (1/)

	if (exists("root:Packages:DebyeWaller:atomicNumber")!=2)
		Init_DebyeWaller()
	endif

	Variable Z = WhichListItem(element,ELEMENT_Symbols)+1	// atomic number
	if (strlen(element)<1 || T_K<0 || Q_Ang<0 || numtype(Z+T_K+Q_Ang))
		if (T_K<0 || numtype(T_K))
			T_K = NumVarOrDefault("root:Packages:DebyeWaller:temperature",300)
		endif
		if (Q_Ang<0 || numtype(Q_Ang))
			Q_Ang = NumVarOrDefault("root:Packages:DebyeWaller:Q_Ang",2.975)
		endif
		if (Z<=0 || numtype(Z))
			Z = NumVarOrDefault("root:Packages:DebyeWaller:atomicNumber",14)
			element = StringFromList(Z-1,ELEMENT_Symbols)
		endif
		Prompt element, "element", popup, ELEMENT_Symbols
		Prompt T_K, "Temperature (K)"
		Prompt Q_Ang, "Length of Q vector (1/)"
		DoPrompt "Debye Waller Factor",element,T_K,Q_Ang
		if (V_flag)
			return NaN
		endif
		Z = WhichListItem(element,ELEMENT_Symbols)+1
	endif
	if (Z<0 || T_K<0 || Q_Ang<0 || numtype(Z+T_K+Q_Ang))
		return NaN
	endif
	Variable amu=Element_amu(Z)			// mass of atom (amu)
	Variable thetaM = Element_DebyeT(Z)	// Debye Temperature (K)
	if (numtype(amu+thetaM))
		print "Do NOT have Debye Temperature for "+Element_names(Z)
		return NaN
	endif
	Variable/G root:Packages:DebyeWaller:temperature = T_K	// save as defaults
	Variable/G root:Packages:DebyeWaller:Q_Ang = Q_Ang
	Variable/G root:Packages:DebyeWaller:atomicNumber = Z

	Variable M = DW_factor_M(T_K,thetaM,Q_Ang,amu)
	printf "%s,      Debye Temperature = %g K     %g amu\r",Element_names(Z),thetaM,amu
	printf "ThetaM / T = %g           M = %g         exp(-2M) = %g\r",thetaM / T_K,M,exp(-2*M)
End


Function MakeDebyeWallerPanel() : Panel
	Init_DebyeWaller()
	if (strlen(WinList("Debye_Waller_Panel", "", "WIN:64"))>1)
		DoWindow/F Debye_Waller_Panel
		return 0
	endif

	NewPanel /W=(7,68,307,200)/K=1
	DoWindow/C Debye_Waller_Panel

	Variable Z = NumVarOrDefault("root:Packages:DebyeWaller:atomicNumber",14)
	Variable T_K = NumVarOrDefault("root:Packages:DebyeWaller:temperature",14)
	Variable Q_Ang = NumVarOrDefault("root:Packages:DebyeWaller:Q_Ang",14)
	String element = StringFromList(Z-1,ELEMENT_Symbols)

	PopupMenu elementNamePopup,pos={8,2},size={97,19}, fSize=12,proc=DWcalc#ElementPopMenuProc,title="element"
	PopupMenu elementNamePopup,mode=1,popvalue=element,value= #("\""+ELEMENT_Symbols+"\"")
	SetVariable temperatureSetVar,pos={8,35},size={230,20},proc=DWcalc#AllSetVarProc,title="Temperature (K)"
	SetVariable temperatureSetVar,fSize=18,limits={0,INF,1},value=_NUM:T_K
	SetVariable QsetVar,pos={130,4},size={107,20},title="Q",fSize=18,proc=DWcalc#AllSetVarProc
	SetVariable QsetVar,limits={0,INF,0.1},value=_NUM:Q_Ang

	ValDisplay val_DebyeTemp,pos={8,70},size={275,20},title="Debye Temperature ="
	ValDisplay val_DebyeTemp,fSize=18,format="%g (K)",frame=0
	ValDisplay val_DebyeTemp,limits={0,0,0},barmisc={0,1000}
	ValDisplay val_DebyeTemp,value= #"300"
	ValDisplay valdisp_M,pos={8,102},size={125,20},title="M =",fSize=18
	ValDisplay valdisp_M,format="%.5f",frame=0,limits={0,0,0},barmisc={0,1000}
	ValDisplay valdisp_exp_M,pos={143,102},size={147,20},title="exp(-M) =",fSize=18
	ValDisplay valdisp_exp_M,format="%.4f",frame=0,limits={0,0,0},barmisc={0,1000}
	reCalcForAllProcs("Debye_Waller_Panel")
	return 0
End
//
Static Function ElementPopMenuProc(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	if (pa.eventCode != 2)			// only on mouse up
		return 0
	endif
	reCalcForAllProcs(pa.win)
	return 0
End
//
Static Function AllSetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	if (sva.eventCode!=1 && sva.eventCode!=2)
		return 0
	endif
	reCalcForAllProcs(sva.win)
	return 0
End
//
Static Function reCalcForAllProcs(win)
	String win
	// set exp(-M) and M based on all other things Q_Ang, temperature

	// Collect the inputs
	ControlInfo/W=$win elementNamePopup
	Variable Z = WhichListItem(S_Value,ELEMENT_Symbols)+1	// atomic number
	ControlInfo/W=$win temperatureSetVar
	Variable T_K = V_Value
	ControlInfo/W=$win QsetVar
	Variable Q_Ang = V_Value

	Variable amu=Element_amu(Z)			// mass of atom (amu)
	Variable thetaM = Element_DebyeT(Z)	// Debye Temperature (K)

	ValDisplay val_DebyeTemp,value=#num2str(thetaM)	// reset displayed Debye Temperature
	if (numtype(amu+thetaM+T_K+Q_Ang))
		return 0
	endif

	Variable/G root:Packages:DebyeWaller:temperature = T_K	// save as defaults
	Variable/G root:Packages:DebyeWaller:Q_Ang = Q_Ang
	Variable/G root:Packages:DebyeWaller:atomicNumber = Z

	Variable M = DW_factor_M(T_K,thetaM,Q_Ang,amu)
	ValDisplay valdisp_M,value=#num2str(M )		// update displayed values in panel
	ValDisplay valdisp_exp_M,value=#num2str(exp(-M) )
End





Function Init_DebyeWaller()
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:DebyeWaller

	if (exists("root:Packages:DebyeWaller:temperature")!=2)
		Variable/G root:Packages:DebyeWaller:temperature = 295
	endif
	if (exists("root:Packages:DebyeWaller:Q_Ang")!=2)
		Variable/G root:Packages:DebyeWaller:Q_Ang = 2.5
	endif
	if (exists("root:Packages:DebyeWaller:atomicNumber")!=2)
		Variable/G root:Packages:DebyeWaller:atomicNumber = 14
	endif

	ElementDataInitPackage()
	return 0
End
